#ifndef MEDIALIBRARY_H
#define MEDIALIBRARY_H

#include <QObject>
#include <QDir>

// Media Information for MELIn packages. v1.0
struct MediaFile {
    quint8 Number; // 0-based contiguous index of the media.
    QString MediaName;// Media name.
    quint32 MediaLength;// Media length (in frames).
};

// Media Library for ELin packages v1.0
struct MediaFolder {
    quint8 m_Id; // Library id.
    QString m_Name;// Library name.
    quint8 m_ElementCount;// Number of elements in the library.
    QList<MediaFile> m_MediaInformation; // Pointer to the Medias Information List of this Library
};


class MediaLibrary : public QObject
{
    Q_OBJECT
public:

    static MediaLibrary *getInstance();

    inline void setPath(QString path) { m_pathmedia = path; rescanMediaLibrary();}

    QString requestNewFile(int folder, int layer);

private:

    explicit MediaLibrary(QObject *parent = 0);
    static MediaLibrary *_instance;
    inline QString getPath () { return m_pathmedia; } //  Get the path to the medias folder tree.

    inline void deleteMediaLibrary() { delete m_media; m_media = NULL; }
    /**
     * Change library/path
     */
    inline void rescanMediaLibrary()
    {
        deleteMediaLibrary();
        initMediaLibrary();
    }

    QList<MediaFolder> *m_media; // Library to save the folders/media libraries and index each media file inside
    QString m_pathmedia; // Path to Medias

    void initMediaLibrary();
    /** Called when there is a change in the channels folder or file
     * this is called. Creates a new source.
    */


    QList<MediaFile> getMediaInformation(QDir dir); // Get all the information of each media file in a dir

signals:
    
public slots:
    
};

#endif // MEDIALIBRARY_H
